import time
from reportlab.lib.enums import TA_JUSTIFY,TA_LEFT
from reportlab.lib.pagesizes import letter
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, Image,PageBreak
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.lib.units import inch
from reportlab.platypus.tables import Table,TableStyle
from reportlab.lib import colors
from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import LETTER
import os

def os_name(details):
    filename = details["sid"] + '_' + details["env"] + '_' +'%s' % details["run_at"] + '.pdf'
    save_path = os.path.join("/var/www/html/caremigration/", "static/", filename)
    doc = SimpleDocTemplate(save_path,pagesize=letter,
                        rightMargin=72,leftMargin=72,
                        topMargin=0,bottomMargin=84)
    doc.title = "Health Check Report"
    return doc

class FooterCanvas(canvas.Canvas):

    def __init__(self, *args, **kwargs):
        canvas.Canvas.__init__(self, *args, **kwargs)
        self.pages = []

    def showPage(self):
        self.pages.append(dict(self.__dict__))
        self._startPage()

    def save(self):
        page_count = len(self.pages)
        for page in self.pages:
            self.__dict__.update(page)
            self.draw_canvas(page_count)
            canvas.Canvas.showPage(self)
        canvas.Canvas.save(self)

    def draw_canvas(self, page_count):
        page = "©2021 Care4SAP. All rights Reserved. | Powered By Canopus GBS.                                                                Page %s of %s" % (self._pageNumber, page_count)
        x = 66
        self.saveState()
        self.setStrokeColorRGB(0, 0, 0)
        self.setLineWidth(0.5)
        self.line(66, 78, LETTER[0] - 66, 78)
        self.setFont('Times-Roman', 10)
        self.drawString(x, 65, page)
        self.restoreState()
styles=getSampleStyleSheet()
styles.add(ParagraphStyle(name='mainheading', leftIndent = 75,textColor= colors.black,fontSize = 28))
styles.add(ParagraphStyle(name='mainheading1', leftIndent = 175,textColor= colors.black,fontSize = 20))
styles.add(ParagraphStyle(name='mainheading2',backColor='#2766A8',borderPadding = (10,0,20,175),leftIndent = 175,textColor= colors.white,fontSize = 20))
styles.add(ParagraphStyle(name='heading', textColor= '#F0AB00',fontSize = 18))
styles.add(ParagraphStyle(name='subheading', textColor= colors.blue,fontSize = 13))
Story=[]
logo1 = "/var/www/html/caremigration/care4s_logo.png"
im1 = Image(logo1, 2*inch, 0.5*inch)
logo2 = "/var/www/html/caremigration/tata_logo.png"
im2 = Image(logo2, 1*inch, 0.5*inch)
chart_style = TableStyle([('ALIGN', (0, 0), (-1, -1), 'LEFT'),
                          ('VALIGN', (0, 0), (-1, -1), 'RIGHT')])
t = Table([[im1, im2]],
                     colWidths=[5 * inch, 1.8* inch],
                     style = chart_style)
 
Story.append(t) 
Story.append(Spacer(1, 30))                         
ptext = "SAP OS Health Check Report"
Story.append(Paragraph(ptext, styles["mainheading"]))
Story.append(Spacer(1, 60))

def osgeneratepdf(details,output,doc):
    # styles.add(ParagraphStyle(name='Justify', alignment=TA_JUSTIFY))

    ptext = '<b>System SID : </b>' + details["sid"]
    Story.append(Paragraph(ptext, styles["Normal"]))
    Story.append(Spacer(1, 12))

    ptext = '<b>Environment : </b>' + details["env"]
    Story.append(Paragraph(ptext, styles["Normal"]))
    Story.append(Spacer(1, 12))

    ptext = '<b>DB System : </b>' + details["dbtype"]
    Story.append(Paragraph(ptext, styles["Normal"]))
    Story.append(Spacer(1, 12))

    ptext = '<b>Log Id : </b>' + details["log_id"]
    Story.append(Paragraph(ptext, styles["Normal"]))
    Story.append(Spacer(1, 12))

    ptext = '<b>Status : </b>' + 'Completed'
    Story.append(Paragraph(ptext, styles["Normal"]))
    Story.append(Spacer(1, 12))

    ptext = '<b>Run By : </b>' + details["run_by"]
    Story.append(Paragraph(ptext, styles["Normal"]))
    Story.append(Spacer(1, 12))

    ptext = 'Run At : '+'%s' % details["run_at"]
    Story.append(Paragraph(ptext, styles["Normal"]))
    Story.append(Spacer(1, 12))

    ptext = '<b><i>This report contains confidential customer data and may be viewed only by SAP DBS employees, authorized SAP partners, and customer employees. Do not distribute it to other parties.</i></b>'
    Story.append(Paragraph(ptext, styles["Normal"]))
    Story.append(Spacer(1, 12))
    Story.append(PageBreak())

    ptext = "SUMMARY"
    Story.append(Paragraph(ptext, styles["mainheading2"]))
    Story.append(Spacer(1, 30)) 

    summary = [['Metrics','Status']]
    for i in output['memory']:
        if i['metric'] == 'Physical':
            total = int(i['value'].split(' ')[0])
        if i['metric'] == 'Free (Value)':
            free = int(i['value'].split(' ')[0])
    if int((free*100)/total)<5:
       summary.append(['Memory Utilization','Red'])
    else:
        summary.append(['Memory Utilization','Green'])        

    for i in output['cpu']:
        if i['metric'] == 'CPU_Utilization':
            used = int(i['value'].split(' ')[0])
    if used>80:
        summary.append(['CPU Utilization','Red'])
    else:
        summary.append(['CPU Utilization','Green'])                    


    temp = []            
    for i in output['fs']:
        if int(i['perc'].split(' ')[0])>85:
            temp.append('red')
        else:
            temp.append('green')            

    if 'red' in temp:
        summary.append(['File System','Red'])
    else:
        summary.append(['File System','Green'])

        t=Table(summary,hAlign='LEFT')
    GRID_STYLE = TableStyle(
        [

            ('GRID',(0,0),(-1,-1),0.5,colors.black),
            ('BACKGROUND', (0, 0), (-1, 0), '#2766A8'),
            ('TEXTCOLOR', (0, 0), (-1, 0), colors.white),

        ]
    )

    for row, values, in enumerate(summary):
        for column, value in enumerate(values):
            if value == "Red":
                GRID_STYLE.add('BACKGROUND', (column, row), (column, row), colors.red)
            if value == "Green":
                GRID_STYLE.add('BACKGROUND', (column, row), (column, row), colors.green)  

    t.setStyle(GRID_STYLE)
    Story.append(t)  
    Story.append(Spacer(1, 12))

    instancelist = [['Instance Name','Instance Number','Features','Port','Status']]
    for i in output['instances']:
        temp = [i['instaname'],i['instanceno'],i['features'],i['http_port'],i['status']]
        instancelist.append(temp)
    t=Table(instancelist,hAlign='LEFT')
    GRID_STYLE = TableStyle(
        [

            ('GRID',(0,0),(-1,-1),0.5,colors.black),
            ('BACKGROUND', (0, 0), (-1, 0), '#2766A8'),
            ('TEXTCOLOR', (0, 0), (-1, 0), colors.white),

        ]
    )
    t.setStyle(GRID_STYLE)
    t._argW[0]=1.36*inch
    t._argW[1]=1.36*inch
    t._argW[2]=2.08*inch
    t._argW[3]=1*inch
    t._argW[4]=1*inch
    Story.append(t)
    Story.append(Spacer(1, 12))

    Story.append(PageBreak())

    ptext = "1 OS"
    Story.append(Paragraph(ptext, styles["heading"]))
    Story.append(Spacer(1, 12)) 
    ptext = "1.1 Memory"
    Story.append(Paragraph(ptext, styles["subheading"]))    
    Story.append(Spacer(1, 12))    


    memlist = [['Type of memory','value']]
    for i in output['memory']:
        temp = [i['metric'],i['value']]
        memlist.append(temp)
    t=Table(memlist,hAlign="LEFT")
    GRID_STYLE = TableStyle(
        [

            ('GRID',(0,0),(-1,-1),0.5,colors.black),
            ('BACKGROUND', (0, 0), (-1, 0), '#2766A8'),
            ('TEXTCOLOR', (0, 0), (-1, 0), colors.white),

        ]
    )
    t.setStyle(GRID_STYLE)

    Story.append(t)
    Story.append(Spacer(1, 12))

    ptext = "1.2 CPU"
    Story.append(Paragraph(ptext, styles["subheading"]))    
    Story.append(Spacer(1, 12))    


    cpulist = [['Type of CPU','value']]
    for i in output['cpu']:
        temp = [i['metric'],i['value']]
        cpulist.append(temp)
    t=Table(cpulist,hAlign="LEFT")
    GRID_STYLE = TableStyle(
        [

            ('GRID',(0,0),(-1,-1),0.5,colors.black),
            ('BACKGROUND', (0, 0), (-1, 0), '#2766A8'),
            ('TEXTCOLOR', (0, 0), (-1, 0), colors.white),

        ]
    )
    t.setStyle(GRID_STYLE)

    Story.append(t)
    Story.append(Spacer(1, 12)) 

    ptext = "1.3 File System"
    Story.append(Paragraph(ptext, styles["subheading"]))    
    Story.append(Spacer(1, 12))    


    fslist = [['File System','Available','Use %']]
    for i in output['fs']:
        temp = [i['filesys'],i['free'],i['perc']]
        fslist.append(temp)
    t=Table(fslist,hAlign="LEFT")
    GRID_STYLE = TableStyle(
        [

            ('GRID',(0,0),(-1,-1),0.5,colors.black),
            ('BACKGROUND', (0, 0), (-1, 0), '#2766A8'),
            ('TEXTCOLOR', (0, 0), (-1, 0), colors.white),

        ]
    )
    t.setStyle(GRID_STYLE)

    Story.append(t)
    Story.append(Spacer(1, 12)) 
    # doc.multiBuild(Story, canvasmaker=FooterCanvas)      
    doc.build(Story)
